
/*
Copyright 2007-2008 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripLoader()
{
	this.hasWhitelist = false;
	this.hasBlacklist = false;
	this.hasStyles = false;

	this.load = function(doc)
	{
		if (!doc /*|| doc.nodeName != "#document"*/) return false;

		var pageAddress = yarip.getAddress(doc, yarip.PAGE_MODE);
		if (!pageAddress) return false;

		var aAddress = yarip.getAddressArray(doc);
		this.doWhitelisting(doc, pageAddress, aAddress);
		this.doBlacklisting(doc, pageAddress, aAddress);
		this.doStyling(doc, pageAddress, aAddress);
		this.doExtensioning(doc, pageAddress, aAddress);

		if (this.hasWhitelist && !this.hasBlacklist) this.removeAllExceptWhitelisted(doc, pageAddress);

		var found = this.hasWhitelist || this.hasBlacklist || this.hasStyles;
		this.hasWhitelist = false;
		this.hasBlacklist = false;
		this.hasStyles = false;
		return found;
	}

	this.doWhitelisting = function(doc, pageAddress, aAddress)
	{
		for (var a in aAddress)
		{
			var list = yarip.map.obj[a].elementWhitelist.obj;
			for (var v in list)
			{
				var element = list[v];
				if (!element.getDeep() && !yarip.equalAddresses(pageAddress, a)) continue;
				if (yarip.whitelistXpath(doc, a, v)) this.hasWhitelist = true;
			}
		}
	}

	this.doBlacklisting = function(doc, pageAddress, aAddress)
	{
		for (var a in aAddress)
		{
			var list = yarip.map.obj[a].elementBlacklist.obj;
			for (var v in list)
			{
				var element = list[v];
				if (!element.getForce() && !element.getDeep() && !yarip.equalAddresses(pageAddress, a)) continue;
				if (!this.hasWhitelist && element.getDOW()) continue;
				if (yarip.blacklistXpath(doc, a, v, false, element.getForce())) this.hasBlacklist = true;
			}
		}
	}

	this.doStyling = function(doc, pageAddress, aAddress)
	{
		for (var a in aAddress)
		{
			var list = yarip.map.obj[a].styles.obj;
			for (var v in list)
			{
				var element = list[v];
				if (!element.getDeep() && !yarip.equalAddresses(pageAddress, a)) continue;
				if (yarip.style(doc, a, v, element.getStyle())) this.hasStyles = true;
			}
		}
	}

	this.doExtensioning = function(doc, pageAddress, aAddress)
	{
		var aExtAddress = yarip.getExtensionAddressArray(aAddress);
		if (!aExtAddress) return;

		this.doWhitelisting(doc, pageAddress, aExtAddress);
		this.doBlacklisting(doc, pageAddress, aExtAddress);
		this.doStyling(doc, pageAddress, aExtAddress);
	}

	this.removeAllExceptWhitelisted = function(doc, pageAddress)
	{
		yarip.whitelistXpath(doc, pageAddress, "/html/head/descendant-or-self::*[not(@status='blacklisted')]"); // whitelist the head and descendants (stylesheets)
		yarip.blacklistXpath(doc, pageAddress, "//*[not(@status='whitelisted')]"); // remove all except whitelisted
	}
}
