
/*
Copyright 2007-2008 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripPage(id, name, elementWhitelist, elementBlacklist, contentWhitelist, contentBlacklist, styles, extension)
{
	this.id = null;
	this.name = "";
	this.elementWhitelist = elementWhitelist ? elementWhitelist : new YaripXPathWhitelist();
	this.elementBlacklist = elementBlacklist ? elementBlacklist : new YaripXPathBlacklist();
	this.contentWhitelist = contentWhitelist ? contentWhitelist : new YaripContentWhitelist();
	this.contentBlacklist = contentBlacklist ? contentBlacklist : new YaripContentBlacklist();
	this.styles = styles ? styles : new YaripStylesList();
	this.extension = extension ? extension : new YaripExtList();

	this.setId = function(id)
	{
		if (id) this.id = "" + id;
		else this.id = "" + yarip.getId();
	}

	this.getId = function()
	{
		return this.id;
	}

	this.setName = function(name)
	{
		if (name) this.name = "" + name;
	}

	this.getName = function()
	{
		return this.name;
	}

	this.clone = function(purge)
	{
		var tmp = new this.constructor(
			purge ? null : this.id,
			this.name,
			this.elementWhitelist.clone(purge),
			this.elementBlacklist.clone(purge),
			this.contentWhitelist.clone(purge),
			this.contentBlacklist.clone(purge),
			this.styles.clone(purge),
			this.extension.clone());
		for (var k in this) if (this[k] && typeof(this[k]) != "object" && !/^(id|name)$/.test(k)) tmp[k] = this[k];
		return tmp;
	}

	this.merge = function(page)
	{
		if (!page) return;
		for (var v in page.elementWhitelist.obj) this.elementWhitelist.add(page.elementWhitelist.obj[v].clone());
		for (var v in page.elementBlacklist.obj) this.elementBlacklist.add(page.elementBlacklist.obj[v].clone());
		for (var v in page.contentWhitelist.obj) this.contentWhitelist.add(page.contentWhitelist.obj[v].clone());
		for (var v in page.contentBlacklist.obj) this.contentBlacklist.add(page.contentBlacklist.obj[v].clone());
		for (var v in page.styles.obj) this.styles.add(page.styles.obj[v].clone());
		for (var v in page.extension.obj) this.extension.add(page.extension.obj[v].clone());
	}

	this.generateXml = function()
	{
		var tmp =
			this.elementWhitelist.generateXml() +
			this.elementBlacklist.generateXml() +
			this.contentWhitelist.generateXml() +
			this.contentBlacklist.generateXml() +
			this.styles.generateXml() +
			this.extension.generateXml();
		if (tmp != "") return "\t<page id=\"" + this.id + "\" name=\"" + this.name + "\">\n" + tmp + "\t</page>\n";
		else return "";
	}

	this.generateCSS = function()
	{
		var tmp =
			this.elementBlacklist.generateCSS() +
			this.styles.generateCSS();
		if (tmp != "")
		{
			var mozDoc = "@-moz-document ";
			if (yarip.domainRegExp.test(this.name)) mozDoc += "domain";
			else if (yarip.uriRegExp.test(this.name)) mozDoc += "url-prefix";
			else return "";
			mozDoc += "(" + this.name + ")\n";
			return mozDoc + "{\n" + tmp + "}\n\n";
		}
		else return "";
	}

	this.setId(id);
	this.setName(name);
}
